
import Dexie from 'dexie';

/**
 * 聊天历史数据库
 * 版本5-6: 修复会话表主键问题
 * - 版本5: 删除旧的会话表结构
 * - 版本6: 使用index_name作为主键重新创建会话表
 * 
 * 注意: Dexie不支持直接更改主键，必须通过删除并重建表的方式实现
 */
export const db = new Dexie('chatHistory');

// 定义数据库表结构和索引
// 版本6：修复主键更改问题
// 版本5：删除旧的会话表
db.version(5).stores({
  conversations: null
}).upgrade(function(trans) {
  // 确保物理删除表
  if (trans.idbtrans.db.objectStoreNames.contains('conversations')) {
    trans.idbtrans.db.deleteObjectStore('conversations');
  }
});

// 版本6：使用新的主键结构重新创建会话表
db.version(6).stores({
  /**
   * 聊天记录表
   * - msg_id: 消息唯一ID (主键)
   * - sequence: 消息序列号，用于排序
   * - [talk_type+receiver_id]: 复合索引，用于快速查询会话消息
   * - created_at: 消息创建时间，用于排序
   * - [talk_type+receiver_id+sequence]: 复合索引，用于高效分页查询
   */
  messages: 'msg_id, sequence, [talk_type+receiver_id], created_at, [talk_type+receiver_id+sequence]',

  /**
   * 会话表
   * - index_name: 主键 (talk_type + '_' + receiver_id)
   * - updated_at: 索引，用于排序
   * - is_top: 索引，用于置顶排序
   */
  conversations: 'index_name, talk_type, receiver_id, updated_at, unread_num, is_top',
});

// 清理旧版本数据
db.on('versionchange', function(event) {
  if (event.oldVersion < 6 && event.newVersion >= 6) {
    console.log('数据库版本升级到6，清理旧数据');
    db.conversations.clear();
    console.log('会话表数据已清理，主键结构已更新');
  }
});

db.on('ready', () => {
  console.log(`数据库已就绪，版本: ${db.verno}`);
});

/** 消息类型常量 */
export const MessageType = {
  TEXT: 1, // 文本消息
  IMAGE: 2, // 图片消息
  FILE: 3, // 文件消息
  AUDIO: 4, // 语音消息
  VIDEO: 5, // 视频消息
  LOCATION: 6, // 位置消息
  CARD: 7, // 名片消息
};

/** 会话类型常量 */
export const TalkType = {
  PRIVATE: 1, // 私聊
  GROUP: 2, // 群聊
};

/**
 * 生成一个简单的UUID
 * @returns {string} UUID
 */
function generateUUID() {
  return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, c => {
    const r = (Math.random() * 16) | 0;
    const v = c === 'x' ? r : (r & 0x3) | 0x8;
    return v.toString(16);
  });
}

// #region 消息操作

/**
 * 添加或更新一条聊天记录
 * @param {object} message - 消息对象
 * @returns {Promise<string>} 消息ID
 */
export async function addMessage(message) {
  try {
    if (!message.msg_id) {
      message.msg_id = generateUUID();
    }
    if (!message.created_at) {
      message.created_at = new Date().toISOString().replace('T', ' ').substring(0, 19);
    }

    // 使用 put 方法，如果主键已存在则更新，否则添加
    await db.messages.put(message);
    return message.msg_id;
  } catch (error) {
    console.error('添加或更新消息失败:', error);
    throw error;
  }
}

/**
 * 批量添加或更新聊天记录
 * @param {Array<object>} messages - 消息对象数组
 * @param {number} talkType - 会话类型
 * @param {number} receiverId - 接收者ID
 * @param {boolean} [updateConversation=true] - 是否更新会话信息
 * @param {string} [sortField='created_at'] - 排序字段
 * @returns {Promise<void>}
 */
export async function batchAddOrUpdateMessages(messages, talkType, receiverId, updateConversation = true, sortField = 'created_at') {
  try {
    if (!Array.isArray(messages) || messages.length === 0) {
      return;
    }

    // 使用批处理优化性能
    return await db.transaction('rw', db.messages, db.conversations, async () => {
      // 预处理消息数据，避免在循环中多次创建对象
      const now = new Date().toISOString().replace('T', ' ').substring(0, 19);
      
      // 使用for循环替代map，减少内存分配
      const messagesToStore = new Array(messages.length);
      for (let i = 0; i < messages.length; i++) {
        const message = messages[i];
        // 确保必要字段存在
        if (!message.msg_id) {
          message.msg_id = generateUUID();
        }
        if (!message.created_at) {
          message.created_at = now;
        }
        // 确保talk_type和receiver_id字段存在
        if (talkType && !message.talk_type) {
          message.talk_type = talkType;
        }
        if (receiverId && !message.receiver_id) {
          message.receiver_id = receiverId;
        }
        messagesToStore[i] = message;
      }

      // 使用bulkPut批量插入/更新，提高性能
      await db.messages.bulkPut(messagesToStore);

      // 只有在需要时才更新会话信息
      if (updateConversation && messagesToStore.length > 0) {
        // 根据排序字段找出最新消息
        let latestMessage;
        if (sortField === 'sequence') {
          // 按sequence排序找出最大的
          latestMessage = messagesToStore.reduce((max, current) => {
            return (current.sequence > (max.sequence || 0)) ? current : max;
          }, messagesToStore[0]);
        } else {
          // 默认按created_at排序
          latestMessage = messagesToStore.reduce((latest, current) => {
            if (!latest.created_at) return current;
            if (!current.created_at) return latest;
            return new Date(current.created_at) > new Date(latest.created_at) ? current : latest;
          }, messagesToStore[0]);
        }
        
        // 异步更新会话最后消息，不阻塞主流程
        updateConversationLastMessage(latestMessage).catch(err => {
          console.error('更新会话最后消息失败:', err);
        });
      }
    });
  } catch (error) {
    console.error('批量添加或更新消息失败:', error);
    throw error;
  }
}

/**
 * 获取指定会话的聊天记录
 * @param {number} talkType - 会话类型 (1:私聊, 2:群聊)
 * @param {number} userId - 当前用户ID
 * @param {number} receiverId - 接收者ID (私聊为对方用户ID，群聊为群ID)
 * @param {number} [limit=30] - 限制返回的记录数量
 * @param {number|null} [maxSequence=null] - 最大sequence值，用于分页加载更早的消息
 * @param {string} [sortField='sequence'] - 排序字段，默认按sequence排序
 * @returns {Promise<Array<object>>} 消息列表 (按sequence升序排列)
 */
export async function getMessages(talkType, userId, receiverId, limit = 30, maxSequence = null, sortField = 'sequence') {
  try {
    // 使用缓存优化重复查询
    const cacheKey = `${talkType}_${receiverId}_${limit}_${maxSequence}_${sortField}`;
    const cachedResult = messageCache.get(cacheKey);
    
    // 如果缓存存在且未过期，直接返回缓存结果
    if (cachedResult && (Date.now() - cachedResult.timestamp < 2000)) { // 2秒缓存
      return cachedResult.data;
    }
    
    let collection;

    // 优化查询策略
    if (maxSequence !== null) {
      // 加载更多：查询 sequence 小于 maxSequence 的消息
      // 使用复合索引优化查询
      collection = db.messages
        .where('[talk_type+receiver_id+sequence]')
        .between([talkType, receiverId, 0], [talkType, receiverId, maxSequence], true, false);
    } else {
      // 首次加载：查询指定会话的所有消息
      // 使用复合索引优化查询
      collection = db.messages.where('[talk_type+receiver_id]').equals([talkType, receiverId]);
    }

    // 优化：根据排序字段选择最优索引
    let messages;
    if (sortField === 'sequence') {
      // 使用sequence字段排序（默认）
      // 1. reverse() - 利用索引倒序排列，获取最新的消息
      // 2. limit() - 限制数量，实现分页
      // 3. toArray() - 执行查询，一次性获取所有数据减少IO操作
      messages = await collection.reverse().limit(limit).toArray();
      // 再次 reverse() - 将获取到的分页消息按时间正序排列，以便于在界面上显示
      messages = messages.reverse();
    } else if (sortField === 'created_at') {
      // 使用created_at字段排序
      messages = await collection.toArray();
      // 在内存中排序，避免数据库排序开销
      messages.sort((a, b) => {
        const dateA = new Date(a.created_at || 0);
        const dateB = new Date(b.created_at || 0);
        return dateA - dateB; // 升序排列
      });
      // 限制返回数量
      messages = messages.slice(-limit);
    } else {
      // 默认排序逻辑
      messages = await collection.reverse().limit(limit).toArray();
      messages = messages.reverse();
    }
    
    // 缓存查询结果
    messageCache.set(cacheKey, {
      data: messages,
      timestamp: Date.now()
    });
    
    return messages;
  } catch (error) {
    console.error('获取消息失败:', error);
    throw error;
  }
}

// 简单的内存缓存实现
const messageCache = new Map();

/**
 * 标记指定会话的所有消息为已读
 * @param {number} talkType - 会话类型
 * @param {number} userId - 当前用户ID
 * @param {number} receiverId - 接收者ID
 * @returns {Promise<number>} 更新的消息数量
 */
export async function markMessagesAsRead(talkType, userId, receiverId) {
  try {
    let query;
    if (talkType === TalkType.PRIVATE) {
      // 私聊：只标记对方发给我的未读消息
      query = db.messages
        .where('[talk_type+receiver_id]')
        .equals([talkType, userId])
        .and(item => item.user_id === receiverId && item.is_read === 0);
    } else {
      // 群聊：标记群里所有非自己的未读消息
      query = db.messages
        .where('[talk_type+receiver_id]')
        .equals([talkType, receiverId])
        .and(item => item.user_id !== userId && item.is_read === 0);
    }
    return await query.modify({ is_read: 1 });
  } catch (error) {
    console.error('批量标记消息已读失败:', error);
    throw error;
  }
}

/**
 * 撤回消息
 * @param {string} msgId - 消息ID
 * @returns {Promise<number>} 更新记录数 (1或0)
 */
export async function revokeMessage(msgId) {
  try {
    return await db.messages.update(msgId, { is_revoke: 1 });
  } catch (error) {
    console.error('撤回消息失败:', error);
    throw error;
  }
}

/**
 * 删除消息
 * @param {string} msgId - 消息ID
 * @returns {Promise<void>}
 */
export async function deleteMessage(msgId) {
  try {
    await db.messages.delete(msgId);
  } catch (error) {
    console.error('删除消息失败:', error);
    throw error;
  }
}

// #endregion 消息操作

// #region 会话操作

/**
 * 添加或更新会话
 * @param {object} conversation - 会话对象
 * @returns {Promise<string>} 会话索引名称
 */
export async function addOrUpdateConversation(conversation) {
  try {
    // 确保 index_name 存在，这是会话表的主键
    if (!conversation.index_name && conversation.talk_type && conversation.receiver_id) {
      conversation.index_name = `${conversation.talk_type}_${conversation.receiver_id}`;
    }
    
    if (!conversation.index_name) {
      throw new Error('无法添加会话：缺少必要的index_name或无法生成');
    }
    
    // 使用 put 方法，如果主键已存在则更新，否则添加
    return await db.conversations.put(conversation);
  } catch (error) {
    console.error('添加或更新会话失败:', error);
    console.error('错误详情:', error.message, error.stack);
    throw error;
  }
}

/**
 * 获取所有会话列表
 * @param {boolean} [includeEmpty=false] - 是否包含没有最后一条消息的会话
 * @returns {Promise<Array<object>>} 会话列表 (按置顶和更新时间排序)
 */
export async function getConversations(includeEmpty = false) {
  try {
    const filterFn = item => !includeEmpty ? (item.msg_text && item.msg_text.length > 0) : true;

    // 分别查询置顶和非置顶会话，以利用索引并优化性能
    const topConversationsPromise = db.conversations
      .where('is_top')
      .equals(1)
      .sortBy('updated_at')
      .then(arr => arr.reverse().filter(filterFn));

    const otherConversationsPromise = db.conversations
      .where('is_top')
      .notEqual(1)
      .sortBy('updated_at')
      .then(arr => arr.reverse().filter(filterFn));

    const [topConversations, otherConversations] = await Promise.all([
      topConversationsPromise,
      otherConversationsPromise,
    ]);

    return [...topConversations, ...otherConversations];
  } catch (error) {
    console.error('获取会话列表失败:', error);
    throw error;
  }
}

/**
 * 获取指定会话
 * @param {number} talkType - 会话类型
 * @param {number} receiverId - 接收者ID
 * @returns {Promise<object|undefined>} 会话对象
 */
export async function getConversation(talkType, receiverId) {
  try {
    const indexName = `${talkType}_${receiverId}`;
    return await db.conversations.get(indexName);
  } catch (error) {
    console.error('获取会话失败:', error);
    throw error;
  }
}

/**
 * 更新会话的未读消息数
 * @param {number} talkType - 会话类型
 * @param {number} receiverId - 接收者ID
 * @param {number|null} unreadNum - 未读消息数。如果为null，则自增1
 * @returns {Promise<number>} 更新的记录数
 */
export async function updateConversationUnreadNum(talkType, receiverId, unreadNum = null) {
  try {
    const indexName = `${talkType}_${receiverId}`;
    const conversation = await db.conversations.get(indexName);

    if (conversation) {
      const newUnreadNum = unreadNum === null ? (conversation.unread_num || 0) + 1 : unreadNum;
      return await db.conversations.update(indexName, { unread_num: newUnreadNum });
    }
    return 0;
  } catch (error) {
    console.error('更新会话未读数失败:', error);
    throw error;
  }
}

/**
 * 清空会话的未读消息数
 * @param {number} talkType - 会话类型
 * @param {number} receiverId - 接收者ID
 * @returns {Promise<number>} 更新的记录数
 */
export function clearConversationUnreadNum(talkType, receiverId) {
  return updateConversationUnreadNum(talkType, receiverId, 0);
}

/**
 * 删除会话及其相关的消息
 * @param {string} indexName - 会话索引名称
 * @param {boolean} [deleteMessages=false] - 是否同时删除相关的消息记录
 * @returns {Promise<void>}
 */
export async function deleteConversation(indexName, deleteMessages = false) {
  try {
    await db.transaction('rw', db.conversations, db.messages, async () => {
      const conversation = await db.conversations.get(indexName);
      if (!conversation) return;

      // 删除会话
      await db.conversations.delete(indexName);

      // 如果需要，删除关联的消息
      if (deleteMessages) {
        const { talk_type, receiver_id } = conversation;
        await db.messages.where({ '[talk_type+receiver_id]': [talk_type, receiver_id] }).delete();
      }
    });
  } catch (error) {
    console.error('删除会话失败:', error);
    throw error;
  }
}

/**
 * 更新会话的最后一条消息摘要
 * @param {object} message - 消息对象
 * @returns {Promise<number>} 更新的记录数
 */
export async function updateConversationLastMessage(message) {
  try {
    const { talk_type, user_id, receiver_id, msg_type } = message;
    const targetReceiverId = talk_type === TalkType.PRIVATE ? (user_id === receiver_id ? user_id : receiver_id) : receiver_id;
    const indexName = `${talk_type}_${targetReceiverId}`;

    const conversation = await db.conversations.get(indexName);
    if (!conversation) return 0;

    let msgText = '';
    switch (msg_type) {
      case MessageType.TEXT: msgText = message.content || ''; break;
      case MessageType.IMAGE: msgText = '[图片]'; break;
      case MessageType.FILE: msgText = '[文件]'; break;
      case MessageType.AUDIO: msgText = '[语音]'; break;
      case MessageType.VIDEO: msgText = '[视频]'; break;
      case MessageType.LOCATION: msgText = '[位置]'; break;
      case MessageType.CARD: msgText = '[名片]'; break;
      default: msgText = '[未知消息]';
    }

    return await db.conversations.update(indexName, {
      msg_text: msgText,
      content: message.content || '',
      updated_at: message.created_at,
    });
  } catch (error) {
    console.error('更新会话最后消息失败:', error);
    throw error;
  }
}

// #endregion 会话操作